' Layout_AutoOpen_Handler:
' Revision: v1.0.2
' (c) 2016 steje
'
' This is a script for written for Directory Opus v12.
' See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

' =====================================================================================
' ======================   EVENT HANDLER: OnAfterFolderChange   =======================
'
' The purpose of this script is to provide a way for users to associate a folder path
' with a layout they would like to call whenever they enter the folder. In most cases
' the existing 'Folder Formats' feature built-in to Opus would be what users would
' most likely use in order to change certain aspects of the lister upon entering a
' particular folder path... But for things like window size and position, Opus Layout
' system does a good job, and this script simply calls a user defined layout when the
' folder it's associated with is opened.
'
' In order to trigger this script to actually do anything, after installing it, users
' must add the full folder path for the folders they want associated for the purposes
' of this script, along with the name of a saved lister Layout they want opened when
' opening the associated folder path. This done via the scripts Configure dialog. The
' Configure dialog will display a FolderLayoutList property for the user to make one
' or more entries to in the following format:
'
' lib://My/projects/opus/scripting && my_Scripting
'
' The scripts behavior can be enabled/disabled by either of the following settings:
'
' Script configuration:          The scripts listing under Preferences
' Script configuration option:   Setting the Always_On option to True/False
' An Opus environment variable:  layout_autoopen
'
' The first setting is controlled by checking or unchecking the scripts listing in Prefs
' The second setting is controlled via the Always_On option under the scripts listing in Prefs
' The third setting can be controlled via any standard button or hotkey in Opus that
' toggles an environment variable of either global, lister or source tab scope, e.g.:
'
'                        @toggle:if $glob:layout_autoopen
'
'                        @ifset:$glob:layout_autoopen
'                        @set glob:layout_autoopen
'
'                        @ifset:else
'                        @set glob:layout_autoopen=true
'
' =====================================================================================

Option Explicit
Dim SCRIPT_DEBUG
SCRIPT_DEBUG = False


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called by Directory Opus to initialize the script
Function OnInit(ByRef initData)

   DOpus.Output("Initializing...")

   ' Provide basic information about the Script
   initData.min_version = "12.2"
   initData.name = "Layout_AutoOpen_Handler"
   initData.desc = "Event Handler for the OnAfterFolderChange event. Used to open a user" &_
                   "defined Layout based on opening a folder path also configured by the" &_
                   "user."
   initData.copyright = "(c) 2016 steje"
   initData.version = "v1.0.2"
   initData.default_enable = True

   '////////////////////////////////////////////////
   ' The following options are user configurable via the Configure button under the scripts listing in Prefs
   '////////////////////////////////////////////////
   ' Set Always_On to True in order to enable script execution without the use of environment variables
   initData.config.Always_On = True
   '////////////////////////////////////////////////
   ' Set DEBUG flag to True in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = False
   '////////////////////////////////////////////////
   ' Set DEBUG_CLEAR flag to True in order to clear log messages from the Opus Output Window between script runs
   initData.config.DEBUG_CLEAR = False
   '////////////////////////////////////////////////
   ' Set FolderLayoutList to define the desired folder and layoput associations using the rules posted to the forum...
   initData.config.FolderLayoutList = DOpus.NewVector("")
   '////////////////////////////////////////////////
   ' Set QualKey to the qualifier key that you want to ENABLE the scripts execution
   initData.config.QualKey = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set QualKey_Disable to the qualifier key that you want to DISABLE the scripts execution
   initData.config.QualKey_Disable = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set descriptions for all script options
   initData.config_desc = DOpus.NewMap("Always_On", "Set this option to True in order to enable script execution without the use of environment variables.", _
                                       "DEBUG", "Set this option to True to enable logging to the Opus script log.", _
                                       "DEBUG_CLEAR", "Set this option to True clear messages logged to the Opus script log between script runs.", _
                                       "FolderLayoutList", "Set this to define the desired folder path and layout associations using the rules posted to the forum...", _
                                       "QualKey", "Set this option to the qualifier key that you want to ENABLE the scripts execution.", _
                                       "QualKey_Disable", "Set this option to the qualifier key that you want to DISABLE the scripts execution.")

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called after a folder has been changed
Function OnAfterFolderChange(ByRef afterFolderChangeData)
   Dim objCmd

   Set objCmd = DOpus.NewCommand
   If (objCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput
   'logMsg("OnAfterFolderChange()")

   logMsg("")
   logMsg("----------------------------------------------------------------")
   logMsg("OnAfterFolderChange:  " & afterFolderChangeData.tab.path)
   logMsg("----------------------------------------------------------------")
   If (executeScript(afterFolderChangeData)) Then
      Call Layout_Open(afterFolderChangeData.tab)
   End If

   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Main worker function of the script for OnAfterFolderChange events
Function Layout_Open(ByRef objSrcTab)
   Dim fFolderLayoutListMatch
   Dim objCmd
   Dim strSrcPath, strLayout, strCommand
   Const vbQuote = """"
   'logMsg("Layout_Open()")

   Layout_Open = False
   fFolderLayoutListMatch = False
   strSrcPath = objSrcTab.path
   strLayout = ""

   logMsg("Source path:  " & DOpus.FSUtil.DisplayName(objSrcTab.path))
   logMsg("")

   fFolderLayoutListMatch = getFolderLayoutListMatch(strSrcPath, strLayout)
   If Not (fFolderLayoutListMatch) Then
      logMsg("No matches found in FolderLayoutList, exit...")
   Else
      logMsg("Opening layout:  " & strLayout)

      Set objCmd = DOpus.NewCommand
      objCmd.SetSourceTab(objSrcTab)
      strCommand = ""

      'Would be nice to have access to the list of saved Layouts so we could check if the one
      'the user specified exists or not before trying to call it...
      strCommand = "Go CURRENT LAYOUT=" & vbQuote & strLayout & vbQuote & " NOSCRIPT"

      'logMsg("  Run:  " & strCommand)
      objCmd.RunCommand(strCommand)
   End If

   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check for various things that determine if the script should run or not
Function executeScript(ByRef folderChangeData)
   Dim QUALKEY
   Dim objCmd
   'logMsg("executeScript()")

   executeScript = False

   ' Check for any of the script options or environment variables that enable the scripts execution
   Set objCmd = DOpus.NewCommand
   If (Script.config.Always_On) Then
      'logMsg("Script is set to 'always on'")
   ElseIf(objCmd.IsSet("$glob:layout_autoopen")) Then
      'logMsg("[layout_autoopen] is set: [global]")
   ElseIf (objCmd.IsSet("$lst:layout_autoopen")) Then
      'logMsg("[layout_autoopen] is set: [lister]")
   ElseIf (objCmd.IsSet("$src:layout_autoopen")) Then
      'logMsg("[layout_autoopen] is set: [source tab]")
   Else
      logMsg("Not enabled, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for any qualifier key that might have been used and recast as int we can test against vector values
   If (folderChangeData.qualifiers = "none") Then
      QUALKEY = 0
   ElseIf (folderChangeData.qualifiers = "shift") Then
      QUALKEY = 1
   ElseIf (folderChangeData.qualifiers = "ctrl") Then
      QUALKEY = 2
   ElseIf (folderChangeData.qualifiers = "alt") Then
      QUALKEY = 3
   ElseIf (folderChangeData.qualifiers = "lwin") Then
      QUALKEY = 4
   ElseIf (folderChangeData.qualifiers = "rwin") Then
      QUALKEY = 5
   End If

   ' Check for DISABLING qualifier key settings
   If ((QUALKEY = Script.config.QualKey_Disable) And (QUALKEY > 0)) Then
      logMsg("The qualifier key to DISABLE was used, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for ENABLING qualifier key settings
   If Not (QUALKEY = Script.config.QualKey) Then
      logMsg("No valid qualifier key was used, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   executeScript = True
   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check if the current path is a child directory of any of the defined folder paths
Function getFolderLayoutListMatch(ByRef strSrcPath, ByRef strLayout)
   Dim i, offset
   Dim strFolderLayoutListEntry, strFolderPath
   'logMsg("getFolderLayoutListMatch()")

   getFolderLayoutListMatch = False

   ' Check if the FolderLayoutList is empty and if so - nothing to do, exit...
   If (Script.config.FolderLayoutList.count = 1) And (Script.config.FolderLayoutList(0) = "") Then
      logMsg("FolderLayoutList is empty, exit...")
      Exit Function
   End If

   ' Parse the FolderLayoutList and determine if the current folder path is listed
   i = 0
   logMsg("Parsing FolderLayoutList...")
   For Each strFolderLayoutListEntry in (Script.config.FolderLayoutList)
      logMsg("  FolderLayoutList(" & i & "):  " & strFolderLayoutListEntry)

      ' Check if the current FolderLayoutListEntry contains the expected delimiter for the folderpath && layout
      offset = 0
      offset = InStr(1, strFolderLayoutListEntry, "&&", vbTextCompare)
      If (offset > 0) Then
         strFolderPath = Trim(Left(strFolderLayoutListEntry, offset-1))
         strLayout = Trim(Mid(strFolderLayoutListEntry, offset+2))
         'logMsg("      " & strFolderPath & " == " & DOpus.FSUtil.DisplayName(strFolderPath))
         'logMsg("      " & strLayout & " == " & strLayout)

         If (DOpus.FSUtil.ComparePath(DOpus.FSUtil.DisplayName(strSrcPath), DOpus.FSUtil.DisplayName(strFolderPath))) Then
            getFolderLayoutListMatch = True
            Exit Function
         End If
      End If

      i = i + 1
   Next

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine that uses DEBUG variables to control whether logging is performed or not
Sub logMsg(ByRef message)

   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) Then DOpus.Output(message)

End Sub
